local mod = ...

--[[

This is the free-form type of pasture (like a wheat farm)
Produces animals which are then processed in sheds or pens

]]



-- Assets -------------------------------------------------------------------------------------
-- Models used for the plantables must have only a single Mesh and a single Material, otherwise planting will crash!
mod:registerAssetId("models/Shed2.fbx/Prefab/ShedPart", "PREFAB_LIVESTOCK_FARM1")

mod:registerAssetId("models/Cow2.fbx/Prefab/Animal", "PREFAB_LIVESTOCK_COW")
mod:registerAssetId("models/Sheep2.fbx/Prefab/Animal", "PREFAB_LIVESTOCK_SHEEP")
mod:registerAssetId("models/Pig2.fbx/Prefab/Animal", "PREFAB_LIVESTOCK_PIG")
mod:registerAssetId("models/Goat2.fbx/Prefab/Animal", "PREFAB_LIVESTOCK_GOAT")

mod:registerAssetId("models/Textures/CowTexture.png", "TEXTURE_COW")
mod:registerAssetId("models/Textures/Colors.png", "TEXTURE_SHEEP")
mod:registerAssetId("models/Textures/PigTexture.png", "TEXTURE_PIG")
mod:registerAssetId("models/Textures/GoatTexture.png", "TEXTURE_GOAT")



-- Construction visual -------------------------------------------------------------------------------------
mod:registerAssetId("models/Shed2.fbx/Prefab/ConstructionSteps", "PREFAB_CONSTR_LIVESTOCK_FARM1")



-- Crop Field Element -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Cow2.fbx/Prefab/Animal", { DataType = "COMP_CROP_FIELD_ELEMENT" })
mod:registerPrefabComponent("models/Sheep2.fbx/Prefab/Animal", { DataType = "COMP_CROP_FIELD_ELEMENT" })
mod:registerPrefabComponent("models/Pig2.fbx/Prefab/Animal", { DataType = "COMP_CROP_FIELD_ELEMENT" })
mod:registerPrefabComponent("models/Goat2.fbx/Prefab/Animal", { DataType = "COMP_CROP_FIELD_ELEMENT" })



-- Plantable -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Cow2.fbx/Prefab/Animal", { DataType = "COMP_PLANTABLE", MinSpaceBetweenPlant = 4.0 })

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_LIVESTOCK_COW",
    Name = "PLANTABLE_LIVESTOCK_COW_NAME",
    PlantPrefab = "PREFAB_LIVESTOCK_COW",
    MinimumScale = 0.70, -- min size at full growth
    MaximumScale = 1.10 -- max size at full growth
})

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_LIVESTOCK_SHEEP",
    Name = "PLANTABLE_LIVESTOCK_SHEEP_NAME",
    PlantPrefab = "PREFAB_LIVESTOCK_SHEEP",
    MinimumScale = 0.70, -- min size at full growth
    MaximumScale = 1.10 -- max size at full growth
})

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_LIVESTOCK_PIG",
    Name = "PLANTABLE_LIVESTOCK_PIG_NAME",
    PlantPrefab = "PREFAB_LIVESTOCK_PIG",
    MinimumScale = 0.70, -- min size at full growth
    MaximumScale = 1.10 -- max size at full growth
})

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_LIVESTOCK_GOAT",
    Name = "PLANTABLE_LIVESTOCK_GOAT_NAME",
    PlantPrefab = "PREFAB_LIVESTOCK_GOAT",
    MinimumScale = 0.70, -- min size at full growth
    MaximumScale = 1.10 -- max size at full growth
})



-- Resource Container -------------------------------------------------------------------------------------
mod:registerPrefabComponent( "models/Cow2.fbx/Prefab/Animal", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "CATTLE_RESOURCE",
    IsReplenishable = true,
    ReplenishDurationInDays = 6,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 3, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false,
    HasMaximumWorkstation = false
})

mod:registerPrefabComponent( "models/Pig2.fbx/Prefab/Animal", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "PIG_RESOURCE",
    IsReplenishable = true,
    ReplenishDurationInDays = 6,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 3, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false,
    HasMaximumWorkstation = false
})

mod:registerPrefabComponent( "models/Sheep2.fbx/Prefab/Animal", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "SHEEP_RESOURCE",
    IsReplenishable = true,
    ReplenishDurationInDays = 6,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 3, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false,
    HasMaximumWorkstation = false
})

mod:registerPrefabComponent( "models/Goat2.fbx/Prefab/Animal", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "GOAT_RESOURCE",
    IsReplenishable = true,
    ReplenishDurationInDays = 6,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 3, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false,
    HasMaximumWorkstation = false
})



-- Zone -------------------------------------------------------------------------------------
mod:register({
    DataType = "ZONE",
    Id = "ZONE_LIVESTOCK_FARM",
    Name = "ZONE_LIVESTOCK_FARM_NAME",
    Description = "ZONE_LIVESTOCK_FARM_DESC",
    Color = {1.57, 1.70, 0.50, 0.9}, -- RGB A = Light greenish yellow
    DesirabilityLayerToShow = nil
})



-- Material -------------------------------------------------------------------------------------
mod:register({
	DataType = "MATERIAL",
	Id = "MATERIAL_LIVESTOCK_COW",
	HasAlphaTest = false,
	AlbedoTexture = "TEXTURE_COW"
})

mod:register({
	DataType = "MATERIAL",
	Id = "MATERIAL_LIVESTOCK_PIG",
	HasAlphaTest = false,
	AlbedoTexture = "TEXTURE_PIG"
})

mod:register({
	DataType = "MATERIAL",
	Id = "MATERIAL_LIVESTOCK_SHEEP",
	HasAlphaTest = false,
	AlbedoTexture = "TEXTURE_SHEEP"
})

mod:register({
	DataType = "MATERIAL",
	Id = "MATERIAL_LIVESTOCK_GOAT",
	HasAlphaTest = false,
	AlbedoTexture = "TEXTURE_GOAT"
})



-- Farm Field Config -------------------------------------------------------------------------------------
mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_COW",
    ZoneType = "ZONE_LIVESTOCK_FARM",
    PlantPrefab = "PREFAB_LIVESTOCK_COW",
    HasPlantingCycle = false,
    IsScaling = false,
    RowDistance = 7.5,
    ItemDistance = 7.5,
    RandomOrientation = {0, 360},
    RandomPositionOffset = {-2, 2},
    SourceMaterial = "MATERIAL_LIVESTOCK_COW",
    GrowingMaterial = "MATERIAL_LIVESTOCK_COW",
    HarvestedMaterial = "MATERIAL_LIVESTOCK_COW",
    GrowingTextureList = {
        "TEXTURE_COW",
    },
    HarvestingTextureList = {
        "TEXTURE_COW",
    },
    AlphaTestThreshold = 0.5
})

mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_PIG",
    ZoneType = "ZONE_LIVESTOCK_FARM",
    PlantPrefab = "PREFAB_LIVESTOCK_PIG",
    HasPlantingCycle = false,
    IsScaling = false,
    RowDistance = 5.5,
    ItemDistance = 5.5,
    RandomOrientation = {0, 360},
    RandomPositionOffset = {-1.25, 1.25},
    SourceMaterial = "MATERIAL_LIVESTOCK_PIG",
    GrowingMaterial = "MATERIAL_LIVESTOCK_PIG",
    HarvestedMaterial = "MATERIAL_LIVESTOCK_PIG",
    GrowingTextureList = {
        "TEXTURE_PIG",
    },
    HarvestingTextureList = {
        "TEXTURE_PIG",
    },
    AlphaTestThreshold = 0.5
})

mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_SHEEP",
    ZoneType = "ZONE_LIVESTOCK_FARM",
    PlantPrefab = "PREFAB_LIVESTOCK_SHEEP",
    HasPlantingCycle = false,
    IsScaling = false,
    RowDistance = 5.5,
    ItemDistance = 5.5,
    RandomOrientation = {0, 360},
    RandomPositionOffset = {-1.25, 1.25},
    SourceMaterial = "MATERIAL_LIVESTOCK_SHEEP",
    GrowingMaterial = "MATERIAL_LIVESTOCK_SHEEP",
    HarvestedMaterial = "MATERIAL_LIVESTOCK_SHEEP",
    GrowingTextureList = {
        "TEXTURE_SHEEP",
    },
    HarvestingTextureList = {
        "TEXTURE_SHEEP",
    },
    AlphaTestThreshold = 0.5
})

mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_GOAT",
    ZoneType = "ZONE_LIVESTOCK_FARM",
    PlantPrefab = "PREFAB_LIVESTOCK_GOAT",
    HasPlantingCycle = false,
    IsScaling = false,
    RowDistance = 5.5,
    ItemDistance = 5.5,
    RandomOrientation = {0, 360},
    RandomPositionOffset = {-1.25, 1.25},
    SourceMaterial = "MATERIAL_LIVESTOCK_GOAT",
    GrowingMaterial = "MATERIAL_LIVESTOCK_GOAT",
    HarvestedMaterial = "MATERIAL_LIVESTOCK_GOAT",
    GrowingTextureList = {
        "TEXTURE_GOAT",
    },
    HarvestingTextureList = {
        "TEXTURE_GOAT",
    },
    AlphaTestThreshold = 0.5
})



-- Sowing Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_SOWING",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
		RightHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_SHEPHERD_STICK"
		},
		IdleAnimation = "GUARD"
	}
})



-- Harvest Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_HARVEST",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
		RightHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_SHEPHERD_STICK"
		},
		IdleAnimation = "GUARD"
	}
})



-- Job: Livestock Farmer -------------------------------------------------------------------------------------

mod:register({
	DataType = "JOB",
	Id = "LIVESTOCKFARMER2_JOB",
	JobName = "LIVESTOCKFARMER2_JOB_NAME",
	JobDescription = "LIVESTOCKFARMER2_JOB_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_FARMER",
	-- DefaultBehavior = "BEHAVIOR_WORK",
    ProductionDelay = 10.0, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
		RightHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_SHEPHERD_STICK"
		},
		IdleAnimation = "GUARD"
	}
})

mod:override({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "LIVESTOCKFARMER2_JOB"
    }
})

mod:override({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "LIVESTOCKFARMER2_JOB"
    }
})

mod:override({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "LIVESTOCKFARMER2_JOB"
    }
})



-- Building Function Assignable  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "BUILDING_FUNCTION_ASSIGNABLE_LIVESTOCK_FARM",
	Name = "BUILDING_FUNCTION_ASSIGNABLE_LIVESTOCK_FARM_NAME",
	AuthorizedFunctionList = "BUILDING_FUNCTION_LIST_LIVESTOCK_FARM"
})



-- Building Function List  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "BUILDING_FUNCTION_LIST_LIVESTOCK_FARM",
	AssetFunctionList = {
		"BUILDING_FUNCTION_LIVESTOCK_FARM_COW",
		"BUILDING_FUNCTION_LIVESTOCK_FARM_PIG",
		"BUILDING_FUNCTION_LIVESTOCK_FARM_SHEEP",
		"BUILDING_FUNCTION_LIVESTOCK_FARM_GOAT",
	}
})



-- Building Function: Cow -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_LIVESTOCK_FARM_COW",
    Name = "BUILDING_FUNCTION_LIVESTOCK_FARM_COW_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "LIVESTOCKFARMER2_JOB", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
	-- 	{ Resource = "WATER", Quantity = 5 },
	-- 	-- { Resource = "DUNG_RESOURCE", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "CATTLE_RESOURCE", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "CATTLE_RESOURCE", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 1}, 
	GrowDuration = {1, 1}, 
	GrowWaitDuration = {1, 1},
	HarvestDuration = {1, 21},
	RestDuration = {21, 21},

    QuantityToHarvestPerStep = 1,
	HarvestStepsCount = 1,
	ResourceValuePerGather = 1,

    QuantityToPlantPerStep = 0,
	PlantingStepsCount = 0,

    CropFieldConfig = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_COW",
    SowingSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_HARVEST"
})



-- Building Function: Pig -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_LIVESTOCK_FARM_PIG",
    Name = "BUILDING_FUNCTION_LIVESTOCK_FARM_PIG_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "LIVESTOCKFARMER2_JOB", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
	-- 	{ Resource = "WATER", Quantity = 5 },
	-- 	-- { Resource = "DUNG_RESOURCE", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "PIG_RESOURCE", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "PIG_RESOURCE", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 1}, 
	GrowDuration = {1, 1}, 
	GrowWaitDuration = {1, 1},
	HarvestDuration = {1, 21},
	RestDuration = {21, 21},

    QuantityToHarvestPerStep = 1,
	HarvestStepsCount = 1,
	ResourceValuePerGather = 1,

    QuantityToPlantPerStep = 0,
	PlantingStepsCount = 0,

    CropFieldConfig = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_PIG",
    SowingSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_HARVEST"
})



-- Building Function: Sheep -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_LIVESTOCK_FARM_SHEEP",
    Name = "BUILDING_FUNCTION_LIVESTOCK_FARM_SHEEP_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "LIVESTOCKFARMER2_JOB", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
	-- 	{ Resource = "WATER", Quantity = 5 },
	-- 	-- { Resource = "DUNG_RESOURCE", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "SHEEP_RESOURCE", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "SHEEP_RESOURCE", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 1}, 
	GrowDuration = {1, 1}, 
	GrowWaitDuration = {1, 1},
	HarvestDuration = {1, 21},
	RestDuration = {21, 21},

    QuantityToHarvestPerStep = 1,
	HarvestStepsCount = 1,
	ResourceValuePerGather = 1,

    QuantityToPlantPerStep = 0,
	PlantingStepsCount = 0,

    CropFieldConfig = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_SHEEP",
    SowingSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_HARVEST"
})



-- Building Function: Goat -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_LIVESTOCK_FARM_GOAT",
    Name = "BUILDING_FUNCTION_LIVESTOCK_FARM_GOAT_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "LIVESTOCKFARMER2_JOB", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
	-- 	{ Resource = "WATER", Quantity = 5 },
	-- 	-- { Resource = "DUNG_RESOURCE", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "GOAT_RESOURCE", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "GOAT_RESOURCE", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 1}, 
	GrowDuration = {1, 1}, 
	GrowWaitDuration = {1, 1},
	HarvestDuration = {1, 21},
	RestDuration = {21, 21},

    QuantityToHarvestPerStep = 1,
	HarvestStepsCount = 1,
	ResourceValuePerGather = 1,

    QuantityToPlantPerStep = 0,
	PlantingStepsCount = 0,

    CropFieldConfig = "FARM_FIELD_CONFIG_LIVESTOCK_FARM_GOAT",
    SowingSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_LIVESTOCK_FARM_HARVEST"
})



-- Building part: Livestock Farm 1 -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
    Id = "BUILDING_PART_LIVESTOCK_FARM1",
    Name = "BUILDING_PART_LIVESTOCK_FARM1_NAME",
    Category = "PASTURE", 
    AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LIVESTOCK_FARM",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {6.0, 4.0}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } 
			}
		}
	},
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_LIVESTOCK_FARM1"
    }, 
	ConstructionVisual = "PREFAB_CONSTR_LIVESTOCK_FARM1",
	Cost = {
		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 5 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 5 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 4 },
		}
	}
})
